# Copyright (c) 2008 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: __init__.py 28752 2008-05-20 09:45:21Z sylvain $

from AccessControl.Permissions import manage_users as ManageUsers
from Products.PluggableAuthService.PluggableAuthService import registerMultiPlugin
from Products.Silva.ExtensionRegistry import extensionRegistry

from plugins import oid
import SilvaOpenIDMember

registerMultiPlugin(oid.OpenIdPlugin.meta_type)

import install

def initialize(context):
    extensionRegistry.register(
        'silva.pas.openid', 'Silva OpenID Support', context, [],
        install, depends_on=('silva.pas.base', 'silva.captcha',))

    context.registerClass(oid.OpenIdPlugin,
                          permission=ManageUsers,
                          constructors=
                          (oid.manage_addOpenIDPluginForm,
                           oid.manage_addOpenIDPlugin),
                          visibility=None,
                          icon="www/openid.png")

    context.registerClass(SilvaOpenIDMember.SilvaOpenIDMember,
                          permission=ManageUsers,
                          constructors=
                          (SilvaOpenIDMember.manage_addOpenIDMemberForm,
                           SilvaOpenIDMember.manage_addOpenIDMember),
                          icon="www/member.png")



# Copyright (c) 2008 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: install.py 28288 2008-04-10 09:34:17Z sylvain $

from zope.interface import alsoProvides, noLongerProvides

from Products.PluggableAuthService.interfaces.plugins import *
from Products.Silva.install import add_fss_directory_view, add_helper
from Products.Silva.install import zpt_add_helper
from silva.pas.base.interfaces import IPASMemberService

from interfaces import IOpenIDAware

def install(root):
    """Installation method for OpenID support
    """
    assert IPASMemberService.providedBy(root.service_members)

    # Change the login page to get a OpenID field
    add_helper(root, 'silva_login_form.html', globals(), zpt_add_helper, 0)
    # Add a OpenID registration page
    add_helper(root, 'silva_openid_register.html', globals(), zpt_add_helper, 0)

    # Register PAS plugins
    registerPASPlugins(root.acl_users)

    # Register views
    registerViews(root.service_view_registry)

    alsoProvides(root.service_members, IOpenIDAware)
    
    
def uninstall(root):
    """Uninstall OpenID support
    """
    assert IPASMemberService.providedBy(root.service_members)
    # FIXME: We should restore the previous login page

    unregisterPASPlugins(root.acl_users)

    # Remove views
    unregisterViews(root.service_view_registry)

    # We remove the registration page.
    root.manage_delObjects(['silva_openid_register.html',])
    noLongerProvides(root.service_members, IOpenIDAware)
    
def is_installed(root):
    return IOpenIDAware.providedBy(root.service_members)


def registerPASPlugins(pas):
    """Register new PAS plugins.
    """
    pas.manage_addProduct['plone.session'].manage_addSessionPlugin('session')
    pas.manage_addProduct['silva.pas.openid'].manage_addOpenIDPlugin('openid')
    pas.manage_addProduct['silva.pas.membership'].manage_addMembershipPlugin('members')

    plugins = pas.plugins
    plugins.activatePlugin(IExtractionPlugin, 'session')
    plugins.activatePlugin(IAuthenticationPlugin, 'session')
    plugins.activatePlugin(ICredentialsResetPlugin, 'session')
    plugins.activatePlugin(ICredentialsUpdatePlugin, 'session')

    plugins.activatePlugin(IExtractionPlugin, 'openid')
    plugins.activatePlugin(IAuthenticationPlugin, 'openid')

    plugins.activatePlugin(IUserEnumerationPlugin, 'members')


def registerViews(reg):
    """Register Views.
    """
    reg.register('edit', 'Silva OpenID Member',
                 ['edit', 'Member', 'SimpleMember'])


def unregisterViews(reg):
    """Unregister Views.
    """
    reg.unregister('edit', 'Silva OpenID Member')


def unregisterPASPlugins(pas):
    """Remove PAS plugins.
    """
    pas.manage_delObjects(['session', 'openid', 'members',])


if __name__ == '__main__':
    print """This module is not an installer. You don't have to run it."""



# Copyright (c) 2008 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: __init__.py 28112 2008-03-19 11:58:42Z sylvain $

# See http://peak.telecommunity.com/DevCenter/setuptools#namespace-packages
try:
    __import__('pkg_resources').declare_namespace(__name__)
except ImportError:
    from pkgutil import extend_path
    __path__ = extend_path(__path__, __name__)



# Copyright (c) 2008 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: views.py 28747 2008-05-20 09:17:01Z sylvain $

from zope.component import getMultiAdapter

from silva.pas.openid.interfaces import IOpenIDAskUserInformation

from Products.Five import BrowserView

from zExceptions import Redirect
from urllib import quote

class StartOpenIDRegistration(BrowserView):

    def _redirect(self, msg):
        root = self.context.get_root()
        args = (root.absolute_url(), quote(msg))
        next_url = '%s/silva_openid_register.html?openid_status=%s' % args
        raise Redirect, next_url

    def __call__(self):
        """Register a new user to use OpenID.
        """

        root = self.context.get_root()

        id = self.request.form.get('__ac_identity_register', '').strip()
        if not id:
            self._redirect("You need to provide an identity.")
            
        captcha = self.request.form.get('__ac_identity_register_captcha', None)
        if not captcha:
            self._redirect("You need to fill the captcha.")

        utility = getMultiAdapter((self.context, self.request), name='captcha')
        if not utility.verify(captcha):
            self._redirect("Error while validating the captcha.")


        self.request.form['__ac_identity_url'] = id # Re-inject the id

        smembers = root.service_members

        if smembers.is_user(id):
            # Authenticate
            # direct lookup ?
            root.acl_users.validate(self.request)
            # We should never be here.
            return

        members = root.Members
        members.manage_addProduct['silva.pas.openid'].manage_addOpenIDMember(id)

        provider = IOpenIDAskUserInformation(self.request)
        provider.require('email')
        provider.optional('nickname')
        provider.optional('fullname')

        # Add some roles ?

        root.acl_users.validate(self.request)
        # We should never be here.


        





# Copyright (c) 2008 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: setup.py 28864 2008-05-27 10:08:20Z sylvain $

from setuptools import setup, find_packages
import os

version = '1.0'

setup(name='silva.pas.openid',
      version=version,
      description="OpenID suport for Silva",
      long_description= open('README.txt').read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Framework :: Zope2",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: BSD License",
        ],
      keywords='pas openid silva',
      author='Sylvain Viollon',
      author_email='info@infrae.com',
      url='http://svn.infrae.com/silva.pas.openid/trunk',
      license='BSD',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['silva', 'silva.pas'],
      include_package_data=True,
      zip_safe=False,
      install_requires=["python-openid >= 2.1",
                        "silva.pas.base",
                        "silva.pas.membership",
                        "silva.captcha",
                        "plone.session",
                        "elementtree",
                        "setuptools"],
      )

